<?php

/**
 * Add the gateway to Jigoshop
 **/
function add_idealcheckoutauthorizedtransfer_gateway($aPaymentGateways) 
{
	$aPaymentGateways[] = 'jigoshop_idealcheckoutauthorizedtransfer';
	
	return $aPaymentGateways;
}

add_filter('jigoshop_payment_gateways', 'add_idealcheckoutauthorizedtransfer_gateway', 20);


class jigoshop_idealcheckoutauthorizedtransfer extends jigoshop_payment_gateway
{
	function __construct()
	{
		$this->id = 'idealcheckoutauthorizedtransfer';	
		$this->paymentcode = 'idealcheckoutauthorizedtransfer';	
		$this->paymentname = 'iDEAL Checkout - Eenmalige machtiging';	
		
		$this->has_fields = true;
		
		$options = Jigoshop_Base::get_options();
      	$this->enabled		= $options->get('idealcheckout_enabled_'.$this->paymentcode);
		$this->title 		= $options->get('idealcheckout_title_'.$this->paymentcode);
        $this->description	= $options->get('idealcheckout_description_'.$this->paymentcode);
		
		add_action('init', array($this, 'check_response'));
		
		
		parent::__construct();
	}
	
	function get_default_options() 
	{
		$aDefaultOptions = array();
		
		// Define the Section name for the Jigoshop_Options
		$aDefaultOptions[] = array( 'name' => $this->paymentname, 'type' => 'title', 'desc' => '' );
		
		// List each option in order of appearance with details
		$aDefaultOptions[] = array(
			'name'		=> 'Enable ' . $this->paymentname,
			'desc' 		=> '',
			'tip' 		=> '',
			'id' 		=> 'idealcheckout_enabled_' . $this->paymentcode,
			'std' 		=> 'yes',
			'type' 		=> 'checkbox',
			'choices'	=> array(
				'no'			=> __('No', 'jigoshop'),
				'yes'			=> __('Yes', 'jigoshop')
			)
		);
		
		$aDefaultOptions[] = array(
			'name'		=> __('Method Title','jigoshop'),
			'desc' 		=> '',
			'tip' 		=> __('This controls the title which the user sees during checkout.','jigoshop'),
			'id' 		=> 'idealcheckout_title_' . $this->paymentcode,
			'std' 		=> $this->paymentname,
			'type' 		=> 'text'
		);

		$aDefaultOptions[] = array(
			'name'		=> 'Description',
			'desc' 		=> '',
			'tip' 		=> __('The description on the bank statement of your customer','jigoshop'),
			'id' 		=> 'idealcheckout_description_' . $this->paymentcode,
			'std' 		=> $this->paymentname,
			'type' 		=> 'text'
		);
		

		
		return $aDefaultOptions;
	}

	public function payment_fields() 
	{			
			
		$sText = $this->paymentname;
		
		echo $sText;
	}
	
	public function process_payment($sOrderId)
	{
		$oOrder = new jigoshop_order($sOrderId);
	
		$sSiteUrl = get_bloginfo('wpurl');

		$aDatabaseSettings = idealcheckout_getDatabaseSettings();

		$sStoreCode = '';
		$sGatewayCode = 'authorizedtransfer';
		// $sLanguageCode = strtolower(substr(get_bloginfo('language'), 0, 2)); // nl|de|en
		$sLanguageCode = ''; // nl, de, en
		$sCountryCode = '';
		$sCurrencyCode = 'EUR';


		$sOrderId = $sOrderId;
		$sOrderCode = idealcheckout_getRandomCode(32);
		$aOrderParams = array('order_id' => $sOrderId, 'order_key' => $oOrder->order_key, 'user_id' => $oOrder->user_id);
		$sTransactionId = idealcheckout_getRandomCode(32);
		$sTransactionCode = idealcheckout_getRandomCode(32);
		$fTransactionAmount = $oOrder->order_total;

		$sTransactionDescription = idealcheckout_getTranslation($sLanguageCode, 'idealcheckout', 'Webshop order #{0}', array($sOrderId));
		$sTransactionPaymentUrl = '';
		
		
		
		$iCheckoutPageId = apply_filters('jigoshop_get_checkout_redirect_page_id', jigoshop_get_page_id('thanks'));
		
		$sTransactionSuccessUrl = add_query_arg('key', $oOrder->order_key, add_query_arg('order', $sOrderId, get_permalink($iCheckoutPageId)));
		$sTransactionPendingUrl = '';
		$sTransactionFailureUrl = $oOrder->get_cancel_order_url();
		$sCallbackUrl = trailingslashit(home_url()) . '?idealcheckout=' . $this->id . '&order_id=' . $sOrderId . '&order_code=' . $sOrderCode; 

		
		$aOrderParams['customer'] = array();
				
		// Shipment data
		list($sStreetName, $sStreetNumber) = idealcheckout_splitAddress($oOrder->order_data['shipping_address_1'] . ' ' . $oOrder->order_data['shipping_address_2']);
	
		$aOrderParams['customer']['shipment_company'] = $oOrder->order_data['shipping_company'];
		$aOrderParams['customer']['shipment_name'] = $oOrder->order_data['shipping_first_name'] . ' ' . $oOrder->order_data['shipping_last_names'];
		$aOrderParams['customer']['shipment_firstname'] = $oOrder->order_data['shipping_first_name'];
		$aOrderParams['customer']['shipment_lastname'] = $oOrder->order_data['shipping_last_name'];
		$aOrderParams['customer']['shipment_mail'] = $oOrder->order_data['shipping_email'];
		$aOrderParams['customer']['shipment_street_name'] = $sStreetName;
		$aOrderParams['customer']['shipment_street_number'] = $sStreetNumber;
		$aOrderParams['customer']['shipment_zip'] = $oOrder->order_data['shipping_postcode'];
		$aOrderParams['customer']['shipment_city'] = $oOrder->order_data['shipping_city'];
		//$aOrderParams['customer']['shipment_country'] = $oOrder->;
		$aOrderParams['customer']['shipment_countrycode'] = $oOrder->order_data['shipping_country'];
		$aOrderParams['customer']['shipment_phone'] = $oOrder->order_data['billing_phone'];

		// Payment data
		list($sStreetName, $sStreetNumber) = idealcheckout_splitAddress($oOrder->shipping_address_1 . ' ' . $oOrder->shipping_address_2);
		
		$aOrderParams['customer']['shipment_name'] = $oOrder->order_data['billing_first_name'] . ' ' . $oOrder->order_data['billing_last_name'];
		$aOrderParams['customer']['payment_firstname'] = $oOrder->order_data['billing_first_name'];
		$aOrderParams['customer']['payment_lastname'] = $oOrder->order_data['billing_last_name'];
		$aOrderParams['customer']['payment_mail'] = $oOrder->order_data['billing_email'];
		$aOrderParams['customer']['payment_company'] = $oOrder->order_data['billing_company'];
		$aOrderParams['customer']['payment_street_name'] = $sStreetName;
		$aOrderParams['customer']['payment_street_number'] = $sStreetNumber;
		$aOrderParams['customer']['payment_zip'] = $oOrder->order_data['billing_postcode'];
		$aOrderParams['customer']['payment_city'] = $oOrder->order_data['billing_city'];
		//$aOrderParams['customer']['payment_country'] = $oOrder->billing_country;
		$aOrderParams['customer']['payment_countrycode'] = $oOrder->order_data['billing_country'];
		$aOrderParams['customer']['payment_phone'] = $oOrder->order_data['billing_phone'];


		
		$iTax = 0;
		
		foreach($oOrder->order_tax as $aTax)
		{
			$iTax += $aTax['amount'];
		}
		
		$aOrderParams['customer']['tax'] = round($iTax * 100.0, 0);
		$aOrderParams['customer']['currency'] = get_option('jigoshop_currency');
		
		$aOrderParams['products'] = array();
		
		if(sizeof($oOrder->items) > 0)
		{
			foreach($oOrder->items as $k => $aProduct)
			{
				if(!empty($aProduct['variation_id'])) 
				{
					$oProduct = new jigoshop_product_variation($aProduct['variation_id']);
				} 
				else 
				{
					$oProduct = new jigoshop_product($aProduct['id']);
				}	

				
				$iProductWeight = $oProduct->get_weight();
				
				$aProduct['name'] = $oProduct->get_title();
													
							
				$fPriceExcl = round($oProduct->get_price_excluding_tax(1), 2) * 100.0;
				$fPriceVat = round($aProduct['taxrate'], 0) * 100.0;
		
				$fVatPercentage = round(($fPriceVat / $fPriceExcl) * 100, 0);
				$fPriceIncl = round($fPriceExcl + $fPriceVat, 2);
				
				$aOrderParams['products'][] = array('record_id' => $aProduct['id'], 'code' => $aProduct['id'], 'description' => $aProduct['name'], 'quantity' => $aProduct['qty'], 'price_incl' => $fPriceIncl, 'price_excl' => $fPriceExcl, 'vat' => $fVatPercentage, 'vat_amount' => $fPriceVat);
			}
		}
		
		
		
		
		// Insert into #_transactions
		$sql = "INSERT INTO `" . $aDatabaseSettings['table'] . "` SET 
`id` = NULL, 
`order_id` = '" . idealcheckout_escapeSql($sOrderId) . "', 
`order_code` = '" . idealcheckout_escapeSql($sOrderCode) . "', 
`order_params` = '" . idealcheckout_escapeSql(idealcheckout_serialize($aOrderParams)) . "', 
`store_code` = " . ($sStoreCode ? "'" . idealcheckout_escapeSql($sStoreCode) . "'" : "NULL") . ", 
`gateway_code` = '" . idealcheckout_escapeSql($sGatewayCode) . "', 
`language_code` = " . ($sLanguageCode ? "'" . idealcheckout_escapeSql($sLanguageCode) . "'" : "NULL") . ", 
`country_code` = " . ($sCountryCode ? "'" . idealcheckout_escapeSql($sCountryCode) . "'" : "NULL") . ", 
`currency_code` = " . ($sCurrencyCode ? "'" . idealcheckout_escapeSql($sCurrencyCode) . "'" : "NULL") . ", 
`transaction_id` = '" . idealcheckout_escapeSql(idealcheckout_getRandomCode(32)) . "', 
`transaction_code` = '" . idealcheckout_escapeSql(idealcheckout_getRandomCode(32)) . "', 
`transaction_params` = NULL, 
`transaction_date` = '" . idealcheckout_escapeSql(time()) . "', 
`transaction_amount` = '" . idealcheckout_escapeSql($fTransactionAmount) . "', 
`transaction_description` = '" . idealcheckout_escapeSql($sTransactionDescription ? $sTransactionDescription : idealcheckout_getRandomCode(32)) . "', 
`transaction_status` = NULL, 
`transaction_url` = NULL, 
`transaction_payment_url` = " . ($sTransactionPaymentUrl ? "'" . idealcheckout_escapeSql($sTransactionPaymentUrl) . "'" : "NULL") . ", 
`transaction_success_url` = " . ($sTransactionSuccessUrl ? "'" . idealcheckout_escapeSql($sTransactionSuccessUrl) . "'" : "NULL") . ", 
`transaction_pending_url` = " . ($sTransactionPendingUrl ? "'" . idealcheckout_escapeSql($sTransactionPendingUrl) . "'" : "NULL") . ", 
`transaction_failure_url` = " . ($sTransactionFailureUrl ? "'" . idealcheckout_escapeSql($sTransactionFailureUrl) . "'" : "NULL") . ", 
`transaction_callback_url` = " . ($sCallbackUrl ? "'" . idealcheckout_escapeSql($sCallbackUrl) . "'" : "NULL") . ", 
`transaction_log` = NULL;";

		idealcheckout_database_query($sql);


		// Return redirect
		return array(
			'result' 	=> 'success',
			'redirect'	=> $sSiteUrl . '/idealcheckout/setup.php?order_id=' . $sOrderId . '&order_code=' . $sOrderCode
		);
	}

		
	public function check_response()
	{		
	
		if(isset($_GET['order_id']) && isset($_GET['order_code']))
		{
			$sOrderId = $_GET['order_id'];
			$sOrderCode = $_GET['order_code'];		
			
			$aDatabaseSettings = idealcheckout_getDatabaseSettings();
			
			// Lookup transaction
			$sql = "SELECT `order_id`, `transaction_status`, `order_params` FROM `" . $aDatabaseSettings['table'] . "` WHERE (`order_id` = '" . idealcheckout_escapeSql($sOrderId) . "') AND (`order_code` = '" . idealcheckout_escapeSql($sOrderCode) . "') ORDER BY `id` DESC LIMIT 1;";
			$aRecord = idealcheckout_database_getRecord($sql);

			if($aRecord && ($oOrder = new jigoshop_order($sOrderId)))
			{
				$aOrderParams = idealcheckout_unserialize($aRecord['order_params']);

				if($oOrder->status !== 'processing' && $oOrder->status !== 'completed') // Already completed
				{
					if(strcasecmp($aRecord['transaction_status'], 'PENDING') === 0) // Set to PENDING
					{
						$oOrder->add_order_note( __($this->paymentname . ' transaction is still Open', 'jigoshop'));
					}
					elseif(strcasecmp($aRecord['transaction_status'], 'SUCCESS') === 0) // Set to SUCCESS
					{
						$oOrder->add_order_note( __($this->paymentname . ' transaction Succes', 'jigoshop') );
						$oOrder->payment_complete();
					}
					else // Set to CANCELLED
					{
						$oOrder->update_status('failed', __($this->paymentname . ' transaction has failed.', 'jigoshop') );
					}
				}			
			}
		}	
	}
		/*
		
		if(!isset($_GET['notify']) && !isset($_GET['callback']))
		{
			wp_redirect( add_query_arg('key', $oOrder->order_key, add_query_arg('order', $sOrderId, get_permalink( apply_filters( 'jigoshop_get_checkout_redirect_page_id', jigoshop_get_page_id('thanks')) ))) ); 
		}
		else
		{
			$sisow = new Sisow($this->merchantId, $this->merchantKey);
			
			$oOrder = new jigoshop_order( $this->order_id );

			if(($ex = $sisow->StatusRequest($this->trxid)) < 0)
			{
				echo $ex . ', ' . $sisow->errorCode . ', ' . $sisow->errorMessage;
				exit;
			}
			else
			{
				if($oOrder->status !== 'processing' && $oOrder->status !== 'completed')
				{
					switch($sisow->status)
					{
						case 'Success':
							$oOrder->add_order_note( __($this->paymentname . ' transaction Succes', 'jigoshop') );
							$oOrder->payment_complete();
							break;
						case 'Reservation':
							$oOrder->add_order_note( __('Reservation made for ' . $this->paymentname, 'jigoshop') );
							$oOrder->payment_complete();
							break;
						case 'Cancelled':
							$oOrder->update_status('failed', __($this->paymentname . ' transaction has failed.', 'jigoshop') );
							break;
						case 'Expired':
							$oOrder->update_status('failed', __($this->paymentname . ' transaction has expired.', 'jigoshop') );
							break;
						case 'Pending':
							$oOrder->add_order_note( __($this->paymentname . ' transaction is still Open', 'jigoshop') );
							break;
					}
				}
			}
		}
		exit;
	}
	
	*/
}


?>